#! /usr/bin/perl
#
# Simple lua module generator
#
# AUTHOR: rd@thc.org
#

my $file = $ARGV[0];

%states = ();
open (DATA, "<$file");

while (<DATA>) {
	chomp;
	next if /^#/ || /^\s*$/;
	if (/^:/) { $header = $_; next; }
	if (/^=/) { $bstate = substr($_, 1); next; }	
	
	($state, $rule) = split /#\s*/;
	push ( @{$states{$state}}, $rule);
}

print "-- $header\n-- Generated by modgen.pl\n\n";
print <<__EOF__;
b = snooze.get()

if b == nil or b == "" then
	snooze.set("$bstate")
	return
end

c = string.sub(b, 1, 4)
b_cut = string.sub(b, 5)

__EOF__

foreach $state (sort keys %states) {
	$odir = "";
	$tab = 1;
	
	printf "function fn_%s()\n", lc($state);
	foreach $rule (sort @{$states{$state}}) {
		($dir, $cond, $nstate, $comment) = split /:\s*/, $rule;
		
		if ($dir ne $odir) {
			if ($dir eq "S") {
				print "\t"x$tab, "x = snooze.get_server()\n\n";
			} else {
				print "\t"x$tab, "x = snooze.get_client()\n\n";
			}
			$odir = $dir;
		}
		
		if ($cond ne "") { 
			print "\t"x$tab, "if ($cond) then\n"; 
			$tab++;
		};
		
		if ($comment ne "") { print "\t"x$tab, "-- TODO: $comment\n"; };
		if ($nstate eq "END") {
			print "\t"x$tab, "snooze.finished()\n";
		} else {
			print "\t"x$tab, "snooze.set (\"$nstate\")\n";
		}
		print "\t"x$tab, "return\n";
		
		if ($cond ne "") { 
			$tab--; 
			print "\t"x$tab, "end\n\n";
		}
	}

	print "end\n\n";
}

print "state_fns = {\n";

foreach $state (sort keys %states) {
	printf "\t[\"%s\"] = fn_%s,\n", $state, lc($state);
}

printf "}\n\nstate_fns[c]()\n";
